declare const schema: {
    type: string;
    required: string[];
    properties: {
        logname: {
            type: string;
            title: string;
            default: string;
        };
        title: {
            type: string;
            title: string;
            maxLength: number;
            default: string;
        };
        typename: {
            type: string;
            title: string;
            default: string;
        };
        bgtime: {
            type: string;
            format: string;
            title: string;
            default: number;
        };
        category: {
            type: string;
            title: string;
        };
        area: {
            type: string;
            title: string;
        };
        priority: {
            type: string;
            title: string;
            enum: number[];
            enumNames: string[];
        };
        classid: {
            type: string;
            title: string;
            enum: number[];
            enumNames: string[];
        };
        kks: {
            type: string;
            title: string;
        };
        description: {
            type: string;
            title: string;
            default: string;
        };
        logid: {
            type: string;
            default: number;
        };
        type: {
            type: string;
            default: number;
        };
        relevant: {
            type: string;
            title: string;
        };
        reporter: {
            type: string;
            title: string;
            default: any;
        };
        acknowledgeclose: {
            type: string;
            title: string;
        };
        kksid: {
            type: string;
            default: number;
        };
    };
};
declare const uiSchema: {
    title: {
        'ui:layout': {
            'grid-area': string;
        };
    };
    classid: {
        'ui:widget': string;
    };
    priority: {
        'ui:widget': string;
    };
    category: {
        'ui:widget': string;
        'ui:options': {
            restLoader: {
                url: string;
                method: string;
                valuePath: string;
                labelPath: string;
            };
        };
    };
    logname: {
        'ui:readonly': boolean;
    };
    typename: {
        'ui:readonly': boolean;
    };
    logid: {
        'ui:widget': string;
    };
    kksid: {
        'ui:widget': string;
    };
    type: {
        'ui:widget': string;
    };
    description: {
        'ui:widget': string;
        'ui:layout': {
            'grid-area': string;
        };
        'ui:view': {
            clamp: number;
        };
    };
    bgtime: {
        'ui:widget': string;
        'ui:disabled': boolean;
    };
    'ui:layout': {
        display: string;
        'grid-auto-rows': string;
        'grid-template-columns': string;
        'grid-column-gap': string;
    };
    'ui:groups': {
        General: {
            properties: string[];
        };
        'Other Configuration': {
            layout: {
                'grid-auto-rows': string;
                'grid-template-columns': string;
                'grid-gap': string;
            };
            properties: string[];
        };
    };
    acknowledgeclose: {
        'ui:widget': string;
        'ui:hideLabel': boolean;
    };
    relevant: {
        'ui:widget': string;
        'ui:hideLabel': boolean;
        'ui:layout': {
            'grid-area': string;
        };
    };
    'ui:hideLabel': boolean;
};
export { schema, uiSchema };
