import { toConsumableArray as _toConsumableArray, typeof as _typeof, toArray as _toArray, objectSpread2 as _objectSpread2 } from '../_virtual/_rollupPluginBabelHelpers.js';

function isEmptyObject(obj) {
  if (!obj) return true;

  for (var i in obj) {
    if (obj[i] !== undefined) {
      return false;
    }
  }

  return true;
}

function isLeaf(node) {
  return node.isLeaf;
}

function schema2tree(name, schema, uiSchema) {
  return [schema2node([name], schema, uiSchema)];
}

var forceChange = function (FORCE_CHANGE_KEY) {
  return function (obj) {
    // workaround for forcing rjsf to rerender because the schema still deep equal to after reordering properties.
    if (FORCE_CHANGE_KEY in obj) {
      delete obj[FORCE_CHANGE_KEY];
    } else {
      obj[FORCE_CHANGE_KEY] = undefined;
    }
  };
}("__".concat(Date.now(), "__"));

try {
  if (process.env.NODE_ENV === 'test') {
    forceChange = function forceChange(a) {
      return a;
    };
  }
} catch (e) {}

function schema2node(path, schema, uiSchema) {
  if (!schema) return null;
  var key = path.join('.');
  var name = path[path.length - 1];

  if (schema.type === 'object') {
    var children = [];

    for (var i in schema.properties) {
      var nextPath = [].concat(_toConsumableArray(path), [i]);
      var child = schema2node(nextPath, schema.properties[i], uiSchema && uiSchema[i]);
      if (child) children.push(child);
    }

    return {
      key: key,
      name: name,
      //path,
      schema: schema,
      uiSchema: uiSchema,
      children: children
    };
  }

  if (schema.type === 'array') {
    var _children;

    if (Array.isArray(schema.items)) {
      var _nextPath = [].concat(_toConsumableArray(path), ['[items]']);

      _children = [{
        key: key + '.[items]',
        name: '[items]',
        //path: nextPath,
        children: schema.items.map(function (a, i) {
          return schema2node([].concat(_toConsumableArray(_nextPath), [i.toString()]), schema.items[i], uiSchema && uiSchema.items && uiSchema.items[i]);
        })
      }];

      if ('additionalItems' in schema) {
        var _nextPath2 = [].concat(_toConsumableArray(path), ['additionalItems']);

        _children.push(schema2node(_nextPath2, schema.additionalItems, uiSchema && uiSchema.additionalItems));
      }
    } else {
      _children = [schema2node([].concat(_toConsumableArray(path), ['items']), schema.items, uiSchema && uiSchema.items)];
    }

    return {
      key: key,
      name: name,
      schema: schema,
      uiSchema: uiSchema,
      children: _children //path,

    };
  }

  return {
    key: key,
    name: name,
    //path,
    schema: schema,
    uiSchema: uiSchema,
    isLeaf: true
  };
}

function getNodeByPath(tree, path) {
  var node;
  var children = tree;
  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    var _loop = function _loop() {
      var i = _step.value;
      node = children.find(function (a) {
        return a.name === i;
      });
      if (!node) return {
        v: null
      };
      children = node.children;
    };

    for (var _iterator = path[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var _ret = _loop();

      if (_typeof(_ret) === "object") return _ret.v;
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return != null) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }

  return node;
}

function updateParentKey(tree, parentKey) {
  return tree.map(function (node) {
    var key = parentKey + '.' + node.name;
    return Object.assign({}, node, {
      key: parentKey + '.' + node.name
    }, node.children && {
      children: updateParentKey(node.children, key)
    });
  });
}

function getNodeParentKey(node) {
  return node.key.substring(0, node.key.length - node.name.length - 1);
}

function updateNodeParentKeyAndName(node, parentKey, name) {
  if (!name) {
    name = node.name;
  }

  if (!parentKey) {
    parentKey = getNodeParentKey(node);
  }

  var newKey = parentKey ? parentKey + '.' + name : name;
  if (node.key === newKey) return node;
  return Object.assign({}, node, {
    name: name,
    key: newKey
  }, node.children && {
    children: updateParentKey(node.children, newKey)
  });
}

function updateSchema(node, newChildren) {
  var newSchema = Object.assign({}, node.schema);

  if (node.schema.type === 'object') {
    var newProperties = {};
    var _iteratorNormalCompletion2 = true;
    var _didIteratorError2 = false;
    var _iteratorError2 = undefined;

    try {
      for (var _iterator2 = newChildren[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
        var child = _step2.value;
        newProperties[child.name] = child.schema;
      }
    } catch (err) {
      _didIteratorError2 = true;
      _iteratorError2 = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion2 && _iterator2.return != null) {
          _iterator2.return();
        }
      } finally {
        if (_didIteratorError2) {
          throw _iteratorError2;
        }
      }
    }

    newSchema.properties = newProperties;
  } else if (node.schema.type === 'array') {
    if (Array.isArray(node.schema.items)) {
      // items is array
      newSchema = Object.assign({}, node.schema);
      var newItemsNode = newChildren.find(function (a) {
        return a.name === '[items]';
      });
      var oldItemsNode = node.children.find(function (a) {
        return a.name === '[items]';
      });

      if (newItemsNode !== oldItemsNode) {
        // items field updated
        if (newItemsNode) {
          newSchema.items = newItemsNode.children.map(function (a) {
            return a.schema;
          });
        } else {
          delete newSchema.items;
        }
      } else {
        // additionalItems field updated
        var newAdditionalItemsNode = newChildren.find(function (a) {
          return a.name === 'additionalItems';
        });

        if (newAdditionalItemsNode) {
          newSchema.additionalItems = newAdditionalItemsNode.schema;
        } else {
          delete newSchema.additionalItems;
        }
      }
    } else {
      // items is not array
      var _newItemsNode = newChildren.find(function (a) {
        return a.name === 'items';
      });

      if (_newItemsNode) {
        newSchema.items = _newItemsNode.schema;
      } else {
        delete newSchema.items;
      }
    }
  }

  return newSchema;
}

function updateUiSchema(node, newChildren) {
  var newUiSchema = {};

  for (var i in node.uiSchema) {
    if (i.startsWith('ui:') || i === 'classNames') {
      newUiSchema[i] = node.uiSchema[i];
    }
  }

  if (node.schema.type === 'object') {
    var _iteratorNormalCompletion3 = true;
    var _didIteratorError3 = false;
    var _iteratorError3 = undefined;

    try {
      for (var _iterator3 = newChildren[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
        var child = _step3.value;

        if (child.uiSchema) {
          newUiSchema[child.name] = child.uiSchema;
        }
      }
    } catch (err) {
      _didIteratorError3 = true;
      _iteratorError3 = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion3 && _iterator3.return != null) {
          _iterator3.return();
        }
      } finally {
        if (_didIteratorError3) {
          throw _iteratorError3;
        }
      }
    }
  } else if (node.schema.type === 'array') {
    if (Array.isArray(node.schema.items)) {
      // items is array
      var newItemsNode = newChildren.find(function (a) {
        return a.name === '[items]';
      });
      var oldItemsNode = node.children.find(function (a) {
        return a.name === '[items]';
      });

      if (newItemsNode !== oldItemsNode) {
        // items field updated
        if (newItemsNode) {
          var empty = true;
          newUiSchema.items = newItemsNode.children.map(function (a) {
            empty = empty && !a.uiSchema;
            return a.uiSchema || null;
          });

          if (empty) {
            delete newUiSchema.items;
          }
        }

        if ('additionalItems' in node.uiSchema) {
          newUiSchema.additionalItems = node.uiSchema.additionalItems;
        }
      } else {
        // additionalItems field updated
        if ('items' in node.uiSchema) {
          newUiSchema.items = node.uiSchema.items;
        }

        var newAdditionalItemsNode = newChildren.find(function (a) {
          return a.name === 'additionalItems';
        });

        if (newAdditionalItemsNode) {
          newUiSchema.additionalItems = newItemsNode.children.map(function (a) {
            return a.schema;
          });
        }
      }
    } else {
      // items is not array
      var _newItemsNode2 = newChildren.find(function (a) {
        return a.name === 'items';
      });

      if (_newItemsNode2 && _newItemsNode2.uiSchema) {
        newUiSchema.items = _newItemsNode2.uiSchema;
      }
    }
  }

  return newUiSchema;
}

function updateNodeByNewChildren(oldNode, newChildren) {
  if (!oldNode.schema) {
    return _objectSpread2({}, oldNode, {
      children: newChildren
    });
  }

  var schema = updateSchema(oldNode, newChildren);
  var uiSchema = updateUiSchema(oldNode, newChildren);
  var newNode = Object.assign({}, oldNode, {
    schema: schema,
    uiSchema: uiSchema,
    children: newChildren
  });

  if (isEmptyObject(uiSchema)) {
    delete newNode.uiSchema;
  }

  return newNode;
}

function updateArrayIndex(tree) {
  return tree.map(function (node, i) {
    return node.name === i.toString() ? node : updateNodeParentKeyAndName(node, null, i.toString());
  });
}

function _removeNodeByPath(tree, _ref) {
  var _ref2 = _toArray(_ref),
      head = _ref2[0],
      tail = _ref2.slice(1);

  var arrayItemsFlag = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;

  if (!head && head !== 0) {
    return tree;
  }

  var removed = false;
  var newTree = [];

  if (!tail.length) {
    var _newTree = tree.filter(function (a) {
      return a.name !== head;
    });

    if (arrayItemsFlag === 2) {
      _newTree = updateArrayIndex(_newTree);
    }

    return _newTree.length === tree.length ? tree : _newTree;
  }

  var _iteratorNormalCompletion4 = true;
  var _didIteratorError4 = false;
  var _iteratorError4 = undefined;

  try {
    for (var _iterator4 = tree[Symbol.iterator](), _step4; !(_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done); _iteratorNormalCompletion4 = true) {
      var node = _step4.value;

      if (node.name !== head) {
        newTree.push(node);
        continue;
      }

      if (node.schema && node.schema.type === 'array' && Array.isArray(node.schema.items)) {
        arrayItemsFlag = 1;
      } else if (arrayItemsFlag === 1 && head === '[items]') {
        arrayItemsFlag = 2;
      } else {
        arrayItemsFlag = 0;
      }

      var newChildren = _removeNodeByPath(node.children, tail, arrayItemsFlag);

      if (newChildren === node.children) {
        // no change
        return tree;
      }

      removed = true;
      var newNode = updateNodeByNewChildren(node, newChildren);
      var newSchema = null;

      if (tail.length === 1 && (newSchema = newNode.schema) && newSchema.required) {
        var newRequired = newSchema.required.filter(function (a) {
          return a !== tail[0];
        });

        if (newRequired.length !== newSchema.required.length) {
          newNode.schema = Object.assign({}, newSchema, {
            required: newRequired.length ? newRequired : undefined
          });
        }
      }

      newTree.push(newNode);
    }
  } catch (err) {
    _didIteratorError4 = true;
    _iteratorError4 = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion4 && _iterator4.return != null) {
        _iterator4.return();
      }
    } finally {
      if (_didIteratorError4) {
        throw _iteratorError4;
      }
    }
  }

  return removed ? newTree : tree;
}

function removeNode(tree, key) {
  return _removeNodeByPath(tree, key.split('.'));
}

function _addNodeByPath(tree, _ref3, position, node2Add) {
  var _ref4 = _toArray(_ref3),
      head = _ref4[0],
      tail = _ref4.slice(1);

  var arrayItemsFlag = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : 0;

  if (!head && head !== 0) {
    return tree;
  }

  var added = false;
  var newTree = [];

  if (!tail.length) {
    tree = tree || [];
    var _added = false;

    for (var i in tree) {
      var cn = tree[i];

      if (cn.name !== head) {
        newTree.push(cn);
        continue;
      }

      _added = true;

      if (position < 0) {
        if (!arrayItemsFlag && tree.find(function (a) {
          return a.name === node2Add.name;
        })) {
          // name already exists;
          return tree;
        }

        newTree.push(updateNodeParentKeyAndName(node2Add, getNodeParentKey(cn)));
      }

      if (position === 0) {
        if (isLeaf(cn)) return tree;

        if (!arrayItemsFlag && cn.children.find(function (a) {
          return a.name === node2Add.name;
        })) {
          // name already exists;
          return tree;
        }

        var newNodeChildren = [].concat(_toConsumableArray(cn.children || []), [updateNodeParentKeyAndName(node2Add, cn.key)]);

        if (arrayItemsFlag === 1) {
          newNodeChildren = updateArrayIndex(newNodeChildren);
        }

        newTree.push(updateNodeByNewChildren(cn, newNodeChildren));
      } else {
        newTree.push(cn);
      }

      if (position > 0) {
        if (!arrayItemsFlag && tree.find(function (a) {
          return a.name === node2Add.name;
        })) {
          // name already exists;
          return tree;
        }

        newTree.push(updateNodeParentKeyAndName(node2Add, getNodeParentKey(cn)));
      }
    }

    if (!_added) return tree;

    if (arrayItemsFlag === 2) {
      return updateArrayIndex(newTree);
    }

    return newTree;
  }

  var _iteratorNormalCompletion5 = true;
  var _didIteratorError5 = false;
  var _iteratorError5 = undefined;

  try {
    for (var _iterator5 = tree[Symbol.iterator](), _step5; !(_iteratorNormalCompletion5 = (_step5 = _iterator5.next()).done); _iteratorNormalCompletion5 = true) {
      var node = _step5.value;

      if (node.name !== head) {
        newTree.push(node);
        continue;
      }

      if (node.schema && node.schema.type === 'array' && Array.isArray(node.schema.items)) {
        arrayItemsFlag = 1;
      } else if (arrayItemsFlag === 1 && head === '[items]') {
        arrayItemsFlag = 2;
      } else {
        arrayItemsFlag = 0;
      }

      var newChildren = _addNodeByPath(node.children, tail, position, node2Add, arrayItemsFlag);

      if (newChildren === node.children) {
        // no change
        return tree;
      }

      added = true;
      newTree.push(updateNodeByNewChildren(node, newChildren));
    }
  } catch (err) {
    _didIteratorError5 = true;
    _iteratorError5 = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion5 && _iterator5.return != null) {
        _iterator5.return();
      }
    } finally {
      if (_didIteratorError5) {
        throw _iteratorError5;
      }
    }
  }

  return added ? newTree : tree;
}

function addNode(tree, targetKey, position, newNode) {
  return _addNodeByPath(tree, targetKey.split('.'), position, newNode);
}

function _moveNodeByPath(tree, _ref7, _ref8, position) {
  var _ref9 = _toArray(_ref7),
      sh = _ref9[0],
      st = _ref9.slice(1);

  var _ref10 = _toArray(_ref8),
      th = _ref10[0],
      tt = _ref10.slice(1);

  var arrayItemsFlag = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : 0;
  if (!sh) return tree;

  if (sh === th && tt.length) {
    var _newTree2 = [];
    var updated = false;
    var _iteratorNormalCompletion6 = true;
    var _didIteratorError6 = false;
    var _iteratorError6 = undefined;

    try {
      for (var _iterator6 = tree[Symbol.iterator](), _step6; !(_iteratorNormalCompletion6 = (_step6 = _iterator6.next()).done); _iteratorNormalCompletion6 = true) {
        var node = _step6.value;

        if (node.name !== sh) {
          _newTree2.push(node);

          continue;
        }

        if (isLeaf(node)) return tree;

        if (node.schema && node.schema.type === 'array' && Array.isArray(node.schema.items)) {
          arrayItemsFlag = 1;
        } else if (arrayItemsFlag === 1 && sh === '[items]') {
          arrayItemsFlag = 2;
        } else {
          arrayItemsFlag = 0;
        }

        var newChildren = _moveNodeByPath(node.children, st, tt, position, arrayItemsFlag);

        if (newChildren === node.children) {
          // no change
          return tree;
        }

        updated = true;

        if (tt.length === 1 && st.length === 1 && node.schema && node.schema.type === 'object') {
          // reorder object properties
          var properties = {};
          var _iteratorNormalCompletion7 = true;
          var _didIteratorError7 = false;
          var _iteratorError7 = undefined;

          try {
            for (var _iterator7 = newChildren[Symbol.iterator](), _step7; !(_iteratorNormalCompletion7 = (_step7 = _iterator7.next()).done); _iteratorNormalCompletion7 = true) {
              var child = _step7.value;
              properties[child.name] = child.schema;
            }
          } catch (err) {
            _didIteratorError7 = true;
            _iteratorError7 = err;
          } finally {
            try {
              if (!_iteratorNormalCompletion7 && _iterator7.return != null) {
                _iterator7.return();
              }
            } finally {
              if (_didIteratorError7) {
                throw _iteratorError7;
              }
            }
          }

          var schema = Object.assign({}, node.schema, {
            properties: properties
          });
          forceChange(schema);

          _newTree2.push(Object.assign({}, node, {
            schema: schema,
            children: newChildren
          }));
        } else {
          _newTree2.push(updateNodeByNewChildren(node, newChildren));
        }
      }
    } catch (err) {
      _didIteratorError6 = true;
      _iteratorError6 = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion6 && _iterator6.return != null) {
          _iterator6.return();
        }
      } finally {
        if (_didIteratorError6) {
          throw _iteratorError6;
        }
      }
    }

    return updated ? _newTree2 : tree;
  }

  if (!st.length && !tt.length) {
    var _newTree3 = [];
    var _updated = false;

    var _node2move = tree.find(function (a) {
      return a.name === sh;
    });

    if (!_node2move) return tree;

    for (var i in tree) {
      var cn = tree[i];
      if (cn.name === sh) continue;

      if (cn.name === th) {
        _updated = true;
        if (position < 0) _newTree3.push(_node2move);

        if (position === 0) {
          if (isLeaf(cn)) return tree;
          var newNodeChildren = [].concat(_toConsumableArray(cn.children || []), [updateNodeParentKeyAndName(_node2move, cn.key)]);

          if (arrayItemsFlag === 1) {
            newNodeChildren = updateArrayIndex(newNodeChildren);
          }

          _newTree3.push(updateNodeByNewChildren(cn, newNodeChildren));
        } else {
          _newTree3.push(cn);
        }

        if (position > 0) _newTree3.push(_node2move);
      } else {
        _newTree3.push(cn);
      }
    }

    if (arrayItemsFlag === 2) {
      _newTree3 = updateArrayIndex(_newTree3);
    }

    return _updated ? _newTree3 : tree;
  }

  var node2move = getNodeByPath(tree, [sh].concat(_toConsumableArray(st)));
  if (!node2move) return tree;

  var tmpTree = _removeNodeByPath(tree, [sh].concat(_toConsumableArray(st)), arrayItemsFlag);

  var newTree = _addNodeByPath(tmpTree, [th].concat(_toConsumableArray(tt)), position, node2move, arrayItemsFlag);

  return newTree === tmpTree ? tree : newTree;
}

function moveNode(tree, sourceKey, targetKey, position) {
  return _moveNodeByPath(tree, sourceKey.split('.'), targetKey.split('.'), position);
}

var allowedUpdatedTypes = ['string', 'number', 'integer', 'boolean'];

function _updateNodeByPath(tree, _ref11, nodeUpdate) {
  var _ref12 = _toArray(_ref11),
      head = _ref12[0],
      tail = _ref12.slice(1);

  var newTree = [];
  var updated = false;
  var _iteratorNormalCompletion8 = true;
  var _didIteratorError8 = false;
  var _iteratorError8 = undefined;

  try {
    for (var _iterator8 = tree[Symbol.iterator](), _step8; !(_iteratorNormalCompletion8 = (_step8 = _iterator8.next()).done); _iteratorNormalCompletion8 = true) {
      var node = _step8.value;

      if (node.name !== head) {
        newTree.push(node);
        continue;
      }

      var newNode = Object.assign({}, node);

      if (!tail.length) {
        var _ret2 = function () {
          var schema = nodeUpdate.schema,
              uiSchema = nodeUpdate.uiSchema,
              name = nodeUpdate.name;
          var nu = Object.assign({}, nodeUpdate);
          delete nu.schema;
          delete nu.uiSchema;
          delete nu.name;
          delete nu.children;
          delete nu.key;
          newNode = Object.assign(newNode, nu);
          var typeUpdated = false;

          if (schema) {
            updated = true;
            var oldSchema = node.schema;
            var newSchema = Object.assign({}, schema);

            if (!newSchema.type) {
              newSchema.type = oldSchema.type;
            } else {
              if (newSchema.type !== oldSchema.type) {
                if (allowedUpdatedTypes.includes(newSchema.type) && allowedUpdatedTypes.includes(oldSchema.type)) {
                  typeUpdated = true;

                  for (var i in newSchema) {
                    if (!['type', 'title', 'description'].includes(i)) {
                      delete newSchema[i];
                    }
                  }
                } else {
                  return {
                    v: tree
                  };
                }
              }
            }

            if (newSchema.type === 'object') {
              newSchema.properties = oldSchema.properties;
            }

            if (newSchema.type === 'array') {
              newSchema.items = oldSchema.items;
              newSchema.additionalItems = oldSchema.items;
            }

            newNode.schema = newSchema;
          }

          if (uiSchema) {
            updated = true;
            var oldUiSchema = node.uiSchema;
            var newUiSchema = {};

            for (var _i in uiSchema) {
              if (_i.startsWith('ui:') || _i === 'classNames') {
                newUiSchema[_i] = uiSchema[_i];
              }
            }

            if (typeUpdated) delete newUiSchema['ui:widget'];

            for (var _i2 in node.uiSchema) {
              if (!(_i2.startsWith('ui:') || _i2 === 'classNames')) {
                newUiSchema[_i2] = oldUiSchema[_i2];
              }
            }

            if (isEmptyObject(newUiSchema)) {
              delete newNode.uiSchema;
            } else {
              newNode.uiSchema = newUiSchema;
            }
          }

          if (name && name !== newNode.name) {
            if (tree.find(function (a) {
              return a.name === name;
            })) return {
              v: tree
            };
            updated = true;
            newNode = updateNodeParentKeyAndName(newNode, null, name);
          }

          newTree.push(newNode);
        }();

        if (_typeof(_ret2) === "object") return _ret2.v;
      } else {
        if (node.children) {
          var newChildren = _updateNodeByPath(node.children, tail, nodeUpdate);

          if (newChildren === node.children) {
            return tree;
          }

          updated = true;

          var _newNode = updateNodeByNewChildren(node, newChildren); // update required


          var newSchema = null;

          if (tail.length === 1 && nodeUpdate.name && (newSchema = _newNode.schema) && newSchema.required && newSchema.required.includes(tail[0])) {
            _newNode.schema = Object.assign({}, newSchema, {
              required: newSchema.required.map(function (a) {
                return a === tail[0] ? nodeUpdate.name : a;
              })
            });
          }

          newTree.push(_newNode);
        } else {
          return tree;
        }
      }
    }
  } catch (err) {
    _didIteratorError8 = true;
    _iteratorError8 = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion8 && _iterator8.return != null) {
        _iterator8.return();
      }
    } finally {
      if (_didIteratorError8) {
        throw _iteratorError8;
      }
    }
  }

  return updated ? newTree : tree;
}

function updateNodeByPath(tree, path, nodeUpdate) {
  return _updateNodeByPath(tree, path, nodeUpdate);
}

function updateNode(tree, targetKey, nodeUpdate) {
  return updateNodeByPath(tree, targetKey.split('.'), nodeUpdate);
}

var SEPERATOR = '_';

function getNodeByRjsfId(tree, rjsfId) {
  var _iteratorNormalCompletion9 = true;
  var _didIteratorError9 = false;
  var _iteratorError9 = undefined;

  try {
    for (var _iterator9 = tree[Symbol.iterator](), _step9; !(_iteratorNormalCompletion9 = (_step9 = _iterator9.next()).done); _iteratorNormalCompletion9 = true) {
      var node = _step9.value;

      if (node && node.name === rjsfId) {
        return node;
      }

      if (rjsfId.startsWith(node.name) && rjsfId[node.name.length] === SEPERATOR) {
        if (node.schema.type === 'array') {
          var _ret3 = function () {
            rjsfId = rjsfId.slice(node.name.length + 1);
            var i = rjsfId.indexOf(SEPERATOR);
            var index = i > 0 ? rjsfId.slice(0, i) : rjsfId;
            if (isNaN(index)) return {
              v: null
            };
            var rest = i > 0 ? rjsfId.slice(i + 1) : null;

            if (Array.isArray(node.schema.items)) {
              if (+index < node.schema.items.length) {
                var n = node.children.find(function (a) {
                  return a.name === '[items]';
                });
                n = n && n.children.find(function (a) {
                  return a.name === index;
                });
                return {
                  v: rest ? n && getNodeByRjsfId(n.children, rjsfId.slice(i + 1)) : n
                };
              } else {
                var _n = node.children.find(function (a) {
                  return a.name === 'additionalItems';
                });

                return {
                  v: rest ? _n && getNodeByRjsfId(_n.children, rest) : _n
                };
              }
            } else {
              var _n2 = node.children.find(function (a) {
                return a.name === 'items';
              });

              return {
                v: rest ? _n2 && getNodeByRjsfId(_n2.children, rjsfId.slice(i + 1)) : _n2
              };
            }
          }();

          if (_typeof(_ret3) === "object") return _ret3.v;
        }

        if (node.children) {
          rjsfId = rjsfId.slice(node.name.length + 1);
          var n = getNodeByRjsfId(node.children, rjsfId);
          if (n) return n;
        }
      }
    }
  } catch (err) {
    _didIteratorError9 = true;
    _iteratorError9 = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion9 && _iterator9.return != null) {
        _iterator9.return();
      }
    } finally {
      if (_didIteratorError9) {
        throw _iteratorError9;
      }
    }
  }

  return null;
}

export { addNode, getNodeByRjsfId, moveNode, removeNode, schema2node, schema2tree, updateNode, updateNodeByPath };
