import React from 'react';
import { typeof as _typeof } from '../../_virtual/_rollupPluginBabelHelpers.js';
import { connect } from 'react-redux';
import { Button } from 'antd';
import JsonEditor from '../JsonEditor.js';

function name2title(name) {
  if (!name) return name;
  var words = name.indexOf('_') >= 0 ? name.split('_') : name.split(/(?=[A-Z])/);
  return words.map(function (a) {
    return a.charAt(0).toUpperCase() + a.slice(1);
  }).join(' ');
}

function json2schema(data, name) {
  var title = name ? name2title(name) : undefined;

  if (typeof data === 'number') {
    return {
      type: Number.isInteger(data) ? 'integer' : 'number',
      title: title
    };
  }

  if (typeof data === 'string') {
    return {
      type: 'string',
      title: title
    };
  }

  if (typeof data === 'boolean') {
    return {
      type: 'boolean',
      title: title
    };
  }

  if (_typeof(data) === 'object') {
    if (Array.isArray(data)) {
      return {
        type: 'array',
        title: title,
        items: json2schema(data[0])
      };
    }

    var properties = {};

    for (var i in data) {
      var s = json2schema(data[i], i);

      if (s) {
        properties[i] = s;
      }
    }

    return {
      type: 'object',
      title: title,
      properties: properties
    };
  }

  return undefined;
}

var FormDataView = connect(function (_ref) {
  var formData = _ref.formData;
  return {
    formData: formData
  };
}, function (dispatch) {
  return {
    setFormData: function setFormData(formData) {
      return dispatch({
        type: 'FORM_DATA_SET',
        payload: formData
      });
    },
    genForm: function genForm(formData) {
      return dispatch({
        type: 'TREE_SET_TREE',
        payload: {
          name: 'root',
          schema: console.log(json2schema(formData, 'form')) || json2schema(formData, 'form'),
          uiSchema: {}
        }
      });
    }
  };
})(function (_ref2) {
  var formData = _ref2.formData,
      setFormData = _ref2.setFormData,
      genForm = _ref2.genForm;
  return React.createElement("div", null, React.createElement(JsonEditor, {
    value: formData,
    onChange: function onChange(value) {
      return setFormData(value);
    },
    autosize: true
  }), React.createElement(Button, {
    type: "primary",
    onClick: function onClick() {
      return genForm(formData);
    }
  }, "Gen Form"), React.createElement(Button, {
    onClick: function onClick() {
      return setFormData({});
    }
  }, "Clear"));
});

export default FormDataView;
