import React, { Component } from 'react';
import { Provider, connect } from 'react-redux';
import { store, persistor } from './store';
import { PersistGate } from 'redux-persist/integration/react';
// import './css';
import { Layout, Tabs, Card } from 'antd';
import Tree from './Tree';
import { FormView, SchemaView, UiSchemaView, FormDataView } from './views';
import NodeEditor from './Editor';
import Toolbar from './Toolbar';
import Settings from './Settings';
import openFormList from './remote/openFormList';
import { schema, uiSchema } from './config';
const { Header, Sider, Content } = Layout;
const { TabPane } = Tabs;
const { dispatch } = store;


class App extends Component {
  state = {
    collapsed: false,
    data: this.props.data,
    modal: null,
  };
  toggle = () => {
    this.setState({
      collapsed: !this.state.collapsed,
    });
  };

  setStore = ({ schema, uiSchema, formData }) => {
    dispatch({
      type: 'TREE_SET_TREE',
      payload: { schema, uiSchema },
    });
    dispatch({
      type: 'FORM_DATA_SET',
      payload: formData,
    });
  }

  UNSAFE_componentWillReceiveProps(nextProps) {
    if (nextProps.data !== this.props.data) {
      if (nextProps.data !== null && nextProps.data !== undefined && this.state.modal === null) {
        let m = openFormList(this.state.data, this.handleSelect);
        this.setState({
          modal: m,
          data: nextProps.data,
        })  
      }
    }
  }

  handleSave = (config) => {
    const { formId, formName } = this.state;
    if (this.state.create) {
      if (this.props.onCreate) {
        this.props.onCreate({ name: config.schema.title, ...config });
      }
    } else {
      if (this.props.onSave) {
        this.props.onSave({ id: formId, name: formName, ...config });
      }
    }

  }

  handleSelect = (config) => {
    const { id, name, schema = { title: 'ofm' }, uiSchema = {}, formData = {} } = config || {};
    this.setStore({ schema, uiSchema, formData });
    this.setState({ create: !id, formId: id, formName: name });
  }

  openModal = () => {
    let m = openFormList(this.state.data, this.handleSelect);
    this.setState({ modal: m });
  }

  componentDidUpdate(prevProps) {
    if (prevProps.data !== this.props.data) {
      this.setState({
        data: this.props.data,
      });
      if (this.props.data) {
        
      }
    }
  }

  render() {
    const { settings } = this.props;
    return (
      <Layout style={{ transform: 'rotate(0)' }}>
        <Sider
          trigger={null}
          collapsible
          collapsed={this.state.collapsed}
          width={settings.leftSiderWidth}
          style={{
            background: '#fff',
            padding: 0,
            overflow: 'auto',
            boxShadow: '0 2px 3px 0 rgba(0, 0, 0, 0.2), 0 2px 3px 0 rgba(0, 0, 0, 0.2)',
            height: '100vh',
            position: 'fixed',
            left: 0,
          }}
        >
          <Tabs defaultActiveKey="0" size="small">
            <TabPane tab="Editor" style={{ padding: '8px' }} key="0">
              <Tree />
            </TabPane>
            <TabPane tab="Settings" style={{ padding: '8px' }} key="1">
              <Settings />
            </TabPane>
          </Tabs>
        </Sider>
        <Layout style={{ marginLeft: settings.leftSiderWidth }}>
          <Header style={{ background: '#fff', padding: 0 }}>
            <Toolbar onOpen={() => this.openModal()} onSave={this.handleSave} />
          </Header>
          <Content style={{ minHeight: 280, padding: '12px 8px', display: 'flex', flexWrap: 'wrap' }}>
            <Card
              bordered={false}
              style={{ width: settings.formWidth, margin: '12px 8px', display: 'inline-block', verticalAlign: 'top' }}
            >
              <FormView />
            </Card>
            {(settings.subViews || []).map((a) => {
              const style = { margin: '12px 8px', width: 400, display: 'inline-block', verticalAlign: 'top', maxHeight: '800px', overflowY: 'scroll' };
              switch (a) {
                case 'schema':
                  return (
                    <Card key="schema" title="Schema" style={style}>
                      <SchemaView />
                    </Card>
                  );
                case 'uiSchema':
                  return (
                    <Card key="uiSchema" title="Ui Schema" style={style}>
                      <UiSchemaView />
                    </Card>
                  );
                case 'formData':
                  return (
                    <Card key="formData" title="Form Data" style={style}>
                      <FormDataView />
                    </Card>
                  );
                default:
                  return <div key="null" />;
              }
            })}
          </Content>
        </Layout>
        <Sider
          width={this.props.activeNodeKey ? settings.rightSiderWidth : 0}
          style={{
            overflow: 'auto',
            background: '#fff',
            boxShadow: '0 2px 3px 0 rgba(0, 0, 0, 0.2), 0 2px 3px 0 rgba(0, 0, 0, 0.2)',
            // position: 'fixed',
            height: '100vh',
            // right: 0,
          }}
        >
          <NodeEditor />
        </Sider>
      </Layout>
    );
  }
}

const AppContainer = connect(({ activeNodeKey, settings }) => ({
  activeNodeKey,
  settings,
}))(App);
export default ({ ...props }) => (
  <Provider store={store}>
    <PersistGate loading={null} persistor={persistor}>
      <AppContainer {...props} />
    </PersistGate>
  </Provider>
);
