const schema = {
  type: 'object',
  required: ['title', 'bgtime', 'area', 'category', 'description'],
  properties: {
    logname: {
      type: 'string',
      title: 'Shift Log Name',
      default: 'SHLUN1 AAAAAAAAAAA BBBBBBBBBBBB CCCCCCCCCCCCCCCC',
    },
    title: {
      type: 'string',
      title: 'Event Title',
      maxLength: 50,
      default: 'SHLUN1',
    },
    typename: {
      type: 'string',
      title: 'Shift Type',
      default: 'shift system1(morning shfit)',
    },
    bgtime: {
      type: 'number',
      format: 'date-time',
      title: 'Time Of Event',
      default: Date.now(),
    },
    category: {
      type: 'number',
      title: 'Event Category',
    },
    area: {
      type: 'string',
      title: 'Plant Area',
    },
    priority: {
      type: 'number',
      title: 'Priority',
      enum: [0, 1, 2],
      enumNames: ['High', 'Medium', 'Low'],
    },
    classid: {
      type: 'number',
      title: 'Class',
      enum: [1, 2, 3],
      enumNames: ['Valve', 'Pump', 'Generator'],
    },
    kks: {
      type: 'string',
      title: 'Affected Plant Item(KKS)',
    },
    description: {
      type: 'string',
      title: 'Description',
      default: 'SHLUN1 AAAAAAAAAAA BBBBBBBBBBBB CCCCCCCCCCCCCCCC SHLUN1 AAAAAAAAAAA BBBBBBBBBBBB CCCCCCCCCCCCCCCC SHLUN1 AAAAAAAAAAA BBBBBBBBBBBB CCCCCCCCCCCCCCCC SHLUN1 AAAAAAAAAAA BBBBBBBBBBBB CCCCCCCCCCCCCCCC SHLUN1 AAAAAAAAAAA BBBBBBBBBBBB CCCCCCCCCCCCCCCC'
    },
    logid: {
      type: 'number',
      default: 215,
    },
    type: {
      type: 'number',
      default: 267,
    },
    relevant: {
      type: 'boolean',
      title: 'Relevant to shift handover',
    },
    reporter: {
      type: 'string',
      title: 'Reported by',
      default: localStorage.userName,
    },
    acknowledgeclose: {
      type: 'boolean',
      title: 'Close when acknowledge in shift handover',
    },
    kksid: {
      type: 'number',
      default: 1,
    },
  },
};

const uiSchema = {
  title: {
    'ui:layout': {
      'grid-area': '1 / 1 / 2 / 3',
    },
  },
  classid: {
    'ui:widget': 'TypeaheadWidget',
  },
  priority: {
    'ui:widget': 'TypeaheadWidget',
  },
  category: {
    'ui:widget': 'TypeaheadWidget',
    'ui:options': {
      restLoader: {
        url: 'http://dpp.siemens.com:9200/gateway/ofm-om/shiftevent/event/category/${formData.logid}',
        method: 'get',
        valuePath: '$[*].id',
        labelPath: '$[*].name',
      },
    },
  },
  logname: {
    'ui:readonly': true,
  },
  typename: {
    'ui:readonly': true,
  },
  logid: {
    'ui:widget': 'hidden',
  },
  kksid: {
    'ui:widget': 'hidden',
  },
  type: {
    'ui:widget': 'hidden',
  },
  description: {
    'ui:widget': 'textarea',
    'ui:layout': {
      'grid-area': '6 / 1 / 7 / 3',
    },
    'ui:view': {
      clamp: 2,
    },
  },
  bgtime: {
    'ui:widget': 'DateTimeWidget',
    'ui:disabled': true,
  },
  'ui:layout': {
    display: 'grid',
    'grid-auto-rows': '84px',
    'grid-template-columns': '1fr 1fr',
    'grid-column-gap': '24px',
  },
  'ui:groups': {
    General: {
      properties: [
        'logname',
        'title',
        'typename',
        'category',
        'bgtime',
        'area',
        'priority',
        'classid',
        'description',
        'plantItem',
        'logid',
        'type',
        'kks',
        'reporter'
      ],
    },
    'Other Configuration': {
      layout: {
        'grid-auto-rows': '24px',
        'grid-template-columns': '1fr',
        'grid-gap': '20px',
      },
      properties: ['acknowledgeclose', 'relevant'],
    },
  },
  acknowledgeclose: {
    'ui:widget': 'CheckboxWidget',
    'ui:hideLabel': true,
  },
  relevant: {
    'ui:widget': 'CheckboxWidget',
    'ui:hideLabel': true,
    'ui:layout': {
      'grid-area': '2 / 1 / 3 / 2',
    },
  },
  'ui:hideLabel': true,
};

export { schema, uiSchema };
